package com.nisum.ascend.inventory.service;

import com.nisum.ascend.inventory.dto.Item;
import com.nisum.ascend.inventory.exception.ResourceNotFoundException;
import com.nisum.ascend.inventory.model.Product;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import com.nisum.ascend.inventory.repository.ProductRepository;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;


@Service
public class ProductService {
    @Autowired
    ProductRepository productRepository;


    public Mono<Product> removeProductBySku(String sku) {
        return productRepository
                .findBySku(sku)
                .flatMap(existingProduct -> productRepository.delete(existingProduct)
                        .then(Mono.just(existingProduct)))
                .switchIfEmpty(Mono.error(new ResourceNotFoundException(HttpStatus.NOT_FOUND, "Product Not Found")));
    }

    public Mono<Product> getProductBySku(String sku){
        return productRepository.findBySku(sku)
                .switchIfEmpty(Mono.error(new ResourceNotFoundException(HttpStatus.NOT_FOUND, "product not found")));
    }

    public Flux<Product> findAllProducts() {
        return productRepository.findAll();
    }

    public Mono<Product> updateProductBySku(String sku,  Product product){
        return productRepository.findBySku(sku)
                .flatMap(dbProduct -> {
                    dbProduct.setUpc(product.getUpc());
                    dbProduct.setProductName(product.getProductName());
                    dbProduct.setProductDescription(product.getProductDescription());
                    dbProduct.setBrand(product.getBrand());
                    dbProduct.setCategory(product.getCategory());
                    dbProduct.setPrice(product.getPrice());
                    dbProduct.setAvailableStock(product.getAvailableStock());
                    dbProduct.setProductImageUrl(product.getProductImageUrl());
                    return productRepository.save(dbProduct);
                });
    }

    public Mono<Product> createProduct(Product product) {
        return productRepository.save(product);
    }


    public Mono<Product> updateProductInventoryBySkuWareHouse(String sku, String status, int itemQuantity) {
        System.out.printf("sku = %s, status = %s, itemquanity = %d \n", sku, status, itemQuantity);

        if (status.equals("CANCELED")) {
             return productRepository.findBySku(sku)
                    .flatMap(dbProduct -> {
                        dbProduct.setAvailableStock(dbProduct.getAvailableStock() + itemQuantity);
                        return productRepository.save(dbProduct);
                    });
        } else if (status.equals("RECEIVED")) {
             return productRepository.findBySku(sku)
                    .flatMap(dbProduct -> {
                        dbProduct.setBlockedStock(dbProduct.getBlockedStock() + itemQuantity);
                        return productRepository.save(dbProduct);
                    });
        } else { // if status.equals("FULFILLED")
             return productRepository.findBySku(sku)
                    .flatMap(dbProduct -> {
                        dbProduct.setAvailableStock(dbProduct.getAvailableStock() - itemQuantity);
                        dbProduct.setFulfilledStock(dbProduct.getFulfilledStock() + itemQuantity);
                        dbProduct.setBlockedStock(dbProduct.getBlockedStock() - itemQuantity);
                        return productRepository.save(dbProduct);
                    });
        }
    }

        public Mono<Product> updateProductInventoryBySkuOrder(String sku, String status, int itemQuantity) {
            System.out.printf("sku = %s, status = %s, itemquanity = %d \n", sku, status, itemQuantity);

            if (status.equals("CANCELED")){
                return productRepository.findBySku(sku)
                        .flatMap(dbProduct -> {
                            dbProduct.setAvailableStock(dbProduct.getAvailableStock() + itemQuantity);
                            dbProduct.setBlockedStock(dbProduct.getBlockedStock() - itemQuantity);
                            return productRepository.save(dbProduct);
                        });
            }
            else if (status.equals("RECEIVED")){
                return productRepository.findBySku(sku)
                        .flatMap(dbProduct -> {
                            dbProduct.setBlockedStock(dbProduct.getBlockedStock() + itemQuantity);
                            return productRepository.save(dbProduct);
                        });
            }
            else { // if status.equals("FULFILLED")
                return productRepository.findBySku(sku)
                        .flatMap(dbProduct -> {
                            return productRepository.save(dbProduct);
                        });
            }
    }


}
